----- Plugin: Bars -----

--- Starts a new timer bar, if the id is the same as one allready running the running one will be replaced with this.
-- @param		id			string		Identification of the bar, used when later interacting with the bar (cancel, paus...)
-- @param		duration	number		The duration the bar should have.
-- @optional	text		string		The text that should be displayed on the bar.
-- @optional	icon 		string|number	The icon that should be displayed on the bar, if icon display is enabled.
-- @optional 	color 		table 		The color the bar should use, if nil the color set in the options will be used. (number indexed color table).
-- @optional 	anchorID 	string 		The ID of the anchor where the bar should pop up at first.
-- @usage RW.Callbacks:Fire("BarStart", "bar1", "This is bar 1")
function BarStart(event, id, duration, text, icon, color, anchorID)
end

--- Stops a running bar
-- @param	id	string	The id of the bar that should be stopped
-- @usage RW.Callbacks:Fire("BarStop", "bar1")
function BarStop(event, id)
end

--- Stop all running bars
-- @usage RW.Callbacks:Fire("BarStopAll")
function BarStopAll(event)
end

--- Pause a running bar
-- @param	id		string	The id of the bar that should be paused
-- @param 	pause 	boolean true means the bar should be paused, false or nil means it should resume if it was paused before.
-- @usage RW.Callbacks:Fire("BarPause", "bar1", true)
function BarPause(event, id, pause)
end

--- Adjust the remaining time of a running bar.
-- @param	id			string	The id of the bar that should be adjusted.
-- @param	timeLeft	number	The time the bar should have left.
-- @usage RW.Callbacks:Fire("BarSetTimeLeft", "bar1", 23)
function BarSetTimeLeft(event, id, timeLeft)
end

---- Triggered by Bars:

--- Triggered by Bars plugin when a bar finishes
-- @param	id		string	Identification of the bar.
-- @param	text 	string	The text originally set on the bear.
-- @param	icon 	string	The icon that was last set on the bar.
function BarFinished(event, id, text, icon)
end

----- End Plugin: Bars -----