local mod = RW:Boss("The Lich King", "Icecrown Citadel", "The Frozen Throne")
local L

mod.BossIcon = [[Interface\Icons\achievement_boss_lichking]]
mod.SubCatIcon = [[Interface\Icons\achievement_dungeon_frozenthrone]]

------------------------------------
-- Variables
local mPhase, tActive, tBerserk

-- Shambling Horror
local tShamblingCD, wShambling, tEnrageCD, tEnrageCast, wEnrage

-- Summon Drudge Ghouls
local tDrudgeCD

-- Infest
local tInfestCD, tInfestCast, wInfest

-- Necrotic Plague
local tNecroticCD, tNecroticDur, wNecrotic, mNecrotic, iNecrotic

-- Soul Reaper
local tReaperCD, tReaperDur, wReaper, wReaperApplied, mReaper

-- Defile
local tDefileCD, wDefileCast, wDefileRun, wDefileClose, wDefileDamage, iDefile, mDefile

-- Valkyr
local tValkyrCD, wValkyr, iValkyr, mValkyred, wValkyred

-- Remorseless Winter
local tRemorselessDur, wRemorseless

-- Raging Spirits
local mRagingSpirit, iRagingSpirit, wRagingSpirit, tRagingSpiritCD

-- Vile Spirits
local tSpiritsCD, tSpiritsAttack, wSpirits

-- Harvest Sooul
local tHarvestCD, tHarvestDur, wHarvest, mHarvest

-- Shadow Trap
local tTrapCD, wTrap, wTrapClose, mTrap, iTrap

-- Cave
local tCaveDur

local RoleBits = RW.Utils.Constants.RoleBits

-----------------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale("enUS")
	if L then
		L.YellTrigger = "So the Light's vaunted justice has finally arrived? Shall I lay down Frostmourne and throw myself at your mercy, Fordring?"
		
		L.defile_on_player = "Defile on me!"
		L.trap_on_player = "TRAP ON ME!"
		
		L.active = " active"
		L.active_opt = "Timer for when the fight starts"
		L.active_grup = "Fight start"
		
		L.spirits_attack = "Spirits Attacking"
		L.spirits_attack_opt = "Timer for when the Vile spirits will attack"
		
		L.plague_hisper = "You have got the plague!!"
		L.plague_opt = "Whisper people that get Necrotic Plague"
		
		L.reaper_applied = "Soul Reaper on %s"
		L.reaper_applied_opt = "Warning when it's applied on someone."
		
		L.defile_onyou = "DEFILE ON YOU!"
		L.defile_onyou_opt = "Warning when you are the target."
		L.defile_damage = "Run: You are standing in defile!"
		L.defile_damage_opt = "Warning when you are standing in it."		
		L.defile_close = "Run: Defile Close!"
		L.defile_close_opt = "Defile target is close to you"
		
		L.trap_close = "Run: Trap Close!"
		L.trap_close_opt = "Trap target is close to you"
		
		L.valkyred = "Val'Kyr'd: >%s<"
		L.valkyred_opt = "Display who gets Val'Kyr'd"
		
		L.you_valkyred = "You have been picked up!"
		L.you_valkyred_opt = "Warning when you get picked up"
		
		L.cave = "Frostmourn phase over"
		L.cave_opt = "Timer untill Frostmourn phase ends"
	end
	L = mod:GetLocale()
	mod.L = L
	mod:SetTrigger(36597, L.YellTrigger)
	mod:SetWin()
	mod:SetDelay(60)
	
	self:AddToCategory("Phase 1", 70372, 70337, 72143)
	self:AddToCategory("Phase 2", 69409, 72762, 71843)
	self:AddToCategory("Phase 3", 68980, 70498)
	self:AddToCategory("Phase Transitions", 74270, 69200)
	
	self:AddBoolOption("WhisperOnPlague", L.plague_opt, nil, false, select(3, GetSpellInfo(70337)))
	
	mPhase 				= self:PhaseMessage()
	tActive				= self:CustomTimer(55, (RW.LBB[self:GetName()] or self:GetName())..L.active, L.active_opt, [[Interface\Icons\achievement_boss_lichking]],  L.active_grup)
	tBerserk			= self:BerserkTimer(900)
	
	-- Shambling Horror
	tShamblingCD		= self:CDTimer(60, 70372)
	tEnrageCD			= self:CDTimer(20, 72143, nil, nil, RoleBits.TANK)
	tEnrageCast			= self:CastTimer(1, 72143, nil, nil, RoleBits.TANK)
	wShambling			= self:Warning(70372)
	wEnrage				= self:CastWarning(72143, nil, nil, false)
	
	-- Summon Drudge Ghouls
	tDrudgeCD			= self:CDTimer(30, 70358, nil, nil, RoleBits.TANK)
	
	-- Infest
	tInfestCD			= self:CDTimer(20, 70541)
	tInfestCast			= self:CastTimer(70541)
	wInfest				= self:Warning(70541)
	
	-- Necrotic Plague
	tNecroticCD			= self:CDTimer(30, 70337)
	tNecroticDur		= self:DurTargetTimer(5, 70337)
	mNecrotic			= self:TargetMessage(70337)
	wNecrotic			= self:RunawayWarning(70337)
	iNecrotic			= self:Icon("STAR", 70337, false, 10, "CIRCLE")
	
	-- Soul Reaper
	tReaperCD			= self:CDTimer(30, 69409, nil, nil, RoleBits.TANK_HEALER)
	tReaperDur			= self:DurTargetTimer(5, 69409, nil, nil, RoleBits.TANK_HEALER)
	wReaper				= self:InformWarning(69409, nil, nil, RoleBits.TANK_HEALER)
	wReaperApplied		= self:CustomWarning(L.reaper_applied, L.reaper_applied_opt, 69409, nil, RoleBits.TANK)
	mReaper				= self:TargetMessage(69409, nil, nil, RoleBits.TANK_HEALER)
	
	-- Defile
	tDefileCD			= self:CDTimer(30, 72762)
	wDefileCast			= self:CastWarning(72762)
	wDefileRun			= self:CustomWarning(L.defile_onyou, L.defile_onyou_opt, 72762, nil, nil, nil, "RW: RunAwayLittleGirl")
	wDefileDamage		= self:CustomWarning(L.defile_damage, L.defile_damage_opt, 72762)
	wDefileClose		= self:CustomWarning(L.defile_close, L.defile_close_opt, 72762, nil, nil, nil, "RW: Bell" )
	iDefile				= self:Icon("SKULL", 72762, false, 6)
	mDefile				= self:TargetMessage(72762)
	
	-- Valkyr
	tValkyrCD			= self:CDTimer(47, 71843)
	wValkyr				= self:Warning(71843)
	wValkyred			= self:CustomWarning(L.you_valkyred, L.you_valkyred_opt, 71843)
	mValkyred			= self:CustomMessage(L.valkyred, L.valkyred_opt, 71843)
	iValkyr				= self:Icon("TRIANGLE", 71843, false, nil, "MOON", "DIAMOND")
	
	-- Remorseless Winter
	tRemorselessDur		= self:DurTimer(62, 74270)
	wRemorseless		= self:RunawayWarning(74270)
	
	-- Raging Spirits
	mRagingSpirit		= self:TargetMessage(69200)
	wRagingSpirit		= self:OnYouWarning(69200)
	iRagingSpirit		= self:Icon("TRIANGLE", 69200, false, 5)
	tRagingSpiritCD		= self:CDTimer(23, 69200)
	
	-- Vile Spirits
	tSpiritsCD			= self:CDTimer(32, 70498)
	tSpiritsAttack		= self:CustomTimer(15, L.spirits_attack, L.spirits_attack_opt, 70498)
	wSpirits			= self:CastWarning(70498)
	
	-- Harvest Soul
	tHarvestCD			= self:CDTimer(75, 68980)
	tHarvestDur			= self:DurTargetTimer(6, 68980)
	wHarvest			= self:CastWarning(68980)
	mHarvest			= self:TargetMessage(68980)
	
	-- Shadow Trap
	tTrapCD				= self:CDTimer(16, 73529)
	wTrap				= self:RunawayWarning(73529)
	wTrapClose			= self:CustomWarning(L.trap_close, L.trap_close_opt, 73529, nil, nil, nil, "RW: Bell")
	mTrap				= self:TargetMessage(73529)
	iTrap				= self:Icon("SKULL", 73529, false, 6)
	
	-- Cave
	tCaveDur			= self:CustomTimer(50, L.cave, L.cave_opt, [[Interface\Icons\Achievement_Zone_Ghostlands]], 68980)
end

-----------------------------------------
-- Locals
local phase
local defileSpamm
local necrotic = GetSpellInfo(70337)
local valkyrSpam
local grabbedByValkyr = {}
local markedValkyrs = {}
local plagueSpam = {}

-----------------------------------------
-- Start
function mod:OnEngage(trigger)
	self:Track("ShamblingHorror", 70372)
	self:Track("Drudge", 70358)
	self:Track("ShamblingEnrage", 72143, 72146, 72147, 72148)
	self:Track("Infest", 70541, 73779, 73780, 73781)
	self:Track("NercroticPlague", 70337, 70338, 73785, 73786, 73787, 73912, 73913, 73914)
	self:Track("SoulReaper", 69409, 73797, 73798, 73799)
	self:Track("Defile", 72762, 72754, 73708, 73709, 73710) -- First is the spell casted by arthas to summon the Defile
	self:Track("Remorseless", 74270, 68981, 72259, 74271, 74272, 74273, 74274, 74275)
	self:Track("Valkyr", 69037)
	self:Track("RagingSpirit", 69200)
	self:Track("Spirits", 70498)
	self:Track("FuryOfFrostmourn", 72350, 70063, 72351)
	self:Track("HarvestSoul", 68980, 74325, 74326, 74327, 74297, 74296, 73654, 74295)
	self:Track("Trap", 73529, 73539, 73540)
	self:Track("Quake", 72262)
	
	self:RegisterEvent("UNIT_AURA")
	
	if  trigger == L.YellTrigger then
		tActive:Start()
		tNecroticCD:Schedule(55.5)
	end
	
	tBerserk:Schedule(55.5)
	
	wipe(plagueSpam)
	wipe(markedValkyrs)
	wipe(grabbedByValkyr)
	valkyrSpam = 0
	defileSpamm = 0
	phase = 1
	
end

function mod:UNIT_AURA(event, uID)
	local debuffed = UnitDebuff(uID, necrotic)
	local name = UnitName(uID)
	if debuffed then
		if not plagueSpam[name] then
			plagueSpam[name] = true
		
			if name == self.pName then
				wNecrotic:Show()
				tNecroticDur:Start(name)
				
			elseif self.Party.group[name] then
				mNecrotic:Show(name)
				tNecroticDur:Start(name)
				if self.db.profile.WhisperOnPlague then
					SendChatMessage(L.plague_hisper, "WHISPER", nil, name)
				end				
			end
			iNecrotic:Show(uID)			
		end
	else
		plagueSpam[name] = nil
	end
end

function mod:ShamblingHorror(event, args)
	if event == "SPELL_SUMMON" then
		tShamblingCD:Start()
		wShambling:Show()
	end
end

function mod:ShamblingEnrage(event, args)
	if event == "SPELL_CAST_START" then
		tEnrageCD:Start()
		tEnrageCast:Start()
		wEnrage:Show()
	end
end

function mod:Drudge(event, args)
	if event == "SPELL_CAST_START" then
		tDrudgeCD:Start()
	end
end

function mod:Infest(event, args)
	if event == "SPELL_CAST_START" then
		tInfestCD:Start()
		tInfestCast:Start()
		wInfest:Show()
	end
end

function mod:NercroticPlague(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tNecroticCD:Start()
	
	elseif event == "SPELL_DISPELL" then
		iNecrotic:Hide(args.dName)
		tNecroticDur:Stop(args.dName)
	end
end

function mod:SoulReaper(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tReaperCD:Start()
	
	elseif event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wReaper:Show()
		else
			wReaperApplied:Show(args.dName)
		end
		tReaperDur:Start(args.dName)
		mReaper:Show(args.dName)
	end
end

local function ScanDefileTarget()
	local target = mod:GetMobTarget(36597)
	if target then
		iDefile:Show(target)
		mDefile:Show(target)
	
		if target == mod.pName then
			SendChatMessage("Defile on "..self.pName.."!", "SAY")
			wDefileRun:Show()
			
		elseif mod.Party.group[target] then
			local uID = mod.Party.group[target].id
			if uID and CheckInteractDistance(uID, 2) then
				wDefileClose:Show()
			end
		end
	end
end

function mod:Defile(event, args)
	if event == "SPELL_CAST_START" then
		tDefileCD:Start()
		wDefileCast:Show()
		self:Schedule("DefileScan", 0.1, ScanDefileTarget)
		
	elseif event == "SPELL_DAMAGE" then
		if args:IsPlayer() and GetTime() - defileSpamm > 2 then
			wDefileDamage:Show()
			defileSpamm = GetTime()
		end
	end
end

local function unregisterUnitAura()
	mod:UnregisterEvent("UNIT_AURA")
end

function mod:Remorseless(event, args)
	if event == "SPELL_CAST_START" then
		wRemorseless:Show()
		tRemorselessDur:Start()
		
		tShamblingCD:Stop()
		tNecroticCD:Stop()
		tValkyrCD:Stop()
		tDefileCD:Stop()
		tReaperCD:Stop()
		tTrapCD:Stop()
		
		tRagingSpiritCD:Start(15)
	end
end

function mod:Quake(event, args)
	if event == "SPELL_CAST_START" then
		phase = phase + 1
		mPhase:Show(phase)
		
		tRagingSpiritCD:Stop()
		
		tDefileCD:Start(38)
		tReaperCD:Start(40)
		
		if phase == 2 then
			tInfestCD:Start(13)
			tValkyrCD:Start(22)
		elseif phase == 3 then
			tHarvestCD:Start(12)
			tSpiritsCD:Start(21)
		end
	end
end

local function valkyrScanner()
	for name, data in mod.Party:PartyIterator() do
		local cID = mod:CUIDFromUID(data.id.."target")
		if cID == 36609 then
			local GUID = UnitGUID(data.id.."target")			
			if not markedValkyrs[GUID] and GetRaidTargetIndex(data.id.."target") == nil then				
				iValkyr:Show(data.id.."target")
				markedValkyrs[GUID] = true
			end
		end
	end
	
	if GetTime() - valkyrSpam < 10 then
		mod:Schedule("ValkyrScan", 0.2, valkyrScanner)
	end
end

function mod:UPDATE_MOUSEOVER_UNIT()
	local cID = mod:CUIDFromUID("mouseover")
	if cID == 36609 then
		local GUID = UnitGUID("mouseover")
		if not markedValkyrs[GUID] and GetRaidTargetIndex("mouseover") == nil then
			iValkyr:Show("mouseover")
			markedValkyrs[GUID] = true
		end
	end

	if GetTime() - valkyrSpam > 10 then
		self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	end
end

local function valkyrGrabScanner(startTime)
	if (GetTime() - startTime) < 10 then
		for name, data in pairs(mod.Party.group) do
			if UnitInVehicle(data.id) and not grabbedByValkyr[name] then
				mValkyred:Show(name)
				grabbedByValkyr[name] = true
				if name == mod.pName then
					SendChatMessage(mod.pName.." picked up!", "SAY")
					wValkyred:Show()
				end
			end
		end
		mod:Schedule("ValkyrGrabScan", 0.5, valkyrGrabScanner, startTime)
	else
		wipe(grabbedByValkyr)
	end
end

function mod:Valkyr(event, args)
	if event == "SPELL_SUMMON" and (GetTime() - valkyrSpam > 2) then
		wValkyr:Show()
		tValkyrCD:Start()
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
		valkyrSpam = GetTime()
		self:Schedule("ValkyrScan", 0.5, valkyrScanner)
		self:Schedule("ValkyrGrabScan", 0.5, valkyrGrabScanner, valkyrSpam)
	end
end

function mod:RagingSpirit(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		if args:IsPlayer() then
			wRagingSpirit:Show()
		end
		mRagingSpirit:Show(args.dName)
		iRagingSpirit:Show(args.dName)
		tRagingSpiritCD:Start()
	end
end

function mod:Spirits(event, args)
	if event == "SPELL_CAST_START" then
		tSpiritsCD:Start()
		tSpiritsAttack:Start()
		wSpirits:Show()
	end
end

local function releaseHold()
	mod:ToggleHold()
end

function mod:FuryOfFrostmourn(event, args)
	if event == "SPELL_CAST_START" then
		self:ToggleHold()
		self:Schedule("ReleaseHold", 180, releaseHold)
		tSpiritsCD:Stop()
		tDefileCD:Stop()
		tHarvestCD:Stop()
		tReaperCD:Stop()
	end
end

function mod:HarvestSoul(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		if self:IsHeroic() then
			tDefileCD:Stop()
			tSpiritsCD:Stop()
			tReaperCD:Stop()
			tCaveDur:Start()
			tHarvestCD:Start(105)
		else
			tHarvestCD:Start()
			tHarvestDur:Start(args.dName)
			mHarvest:Show(args.dName)
		end
		wHarvest:Show()
	end
end

local function trapScanner()
	local target = mod:GetMobTarget(36597)
	if target then
		iTrap:Show(target)
		mTrap:Show(target)
	
		if target == mod.pName then
			SendChatMessage("Shadow Trap on "..self.pName.."!", "SAY")
			wTrap:Show()
			
		elseif mod.Party.group[target] then
			local uID = mod.Party.group[target].id
			if uID and CheckInteractDistance(uID, 2) then
				wTrap:Show()
			end
		end
	end
end

local function trapScan()
	local target = mod:GetMobTarget(36597)
	if target then
		iTrap:Show(target)
		mTrap:Show(target)
	
		if target == mod.pName then
			SendChatMessage("Shadow Trap on "..self.pName.."!", "SAY")
			wTrap:Show()
			
		elseif mod.Party.group[target] then
			local uID = mod.Party.group[target].id
			if uID and CheckInteractDistance(uID, 2) then
				wTrapClose:Show()
			end
		end
	end
end

function mod:Trap(event, args)
	if event == "SPELL_CAST_START" or event == "SPELL_CAST_SUCCESS" then
		tTrapCD:Start()
		
		self:Schedule("trapscan", 0.1, trapScan)
	end
end